@@ -7,6 +7,8 @@
 #[cfg(feature = "logging")]
 use crate::log::trace;
 
+use crate::error::{e_fmt, Error, Result};
+
 use std::{
     any::Any,
     cmp,
@@ -1741,10 +1743,10 @@ impl DnsIncoming {
 
     fn read_header(&mut self) -> Result<()> {
         if self.data.len() < MSG_HEADER_LEN {
-            return Err(Error::Msg(format!(
+            return Err(e_fmt!(
                 "DNS incoming: header is too short: {} bytes",
                 self.data.len()
-            )));
+            ));
         }
 
         let data = &self.data[0..];
@@ -2188,32 +2190,3 @@ const fn get_expiration_time(created: u64, ttl: u32, percent: u32) -> u64 {
     // ttl * 1000 * (percent / 100) => ttl * percent * 10
     created + (ttl * percent * 10) as u64
 }
-
-/// A basic error type from this library.
-#[derive(Clone, Debug, PartialEq, Eq)]
-#[non_exhaustive]
-pub enum Error {
-    /// Like a classic EAGAIN. The receiver should retry.
-    Again,
-
-    /// A generic error message.
-    Msg(String),
-
-    /// Error during parsing of ip address
-    ParseIpAddr(String),
-}
-
-impl fmt::Display for Error {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        match self {
-            Self::Msg(s) => write!(f, "{}", s),
-            Self::ParseIpAddr(s) => write!(f, "parsing of ip addr failed, reason: {}", s),
-            Self::Again => write!(f, "try again"),
-        }
-    }
-}
-
-impl std::error::Error for Error {}
-
-/// One and only `Result` type from this library crate.
-pub type Result<T> = core::result::Result<T, Error>;
