@@ -75,6 +75,9 @@ impl ServiceInfo {
     /// - `Option<HashMap<String, String>>`
     /// - slice of tuple: `&[(K, V)]` where `K` and `V` are [`std::string::ToString`].
     ///
+    /// Note: The maximum length of a single property string is `255`, Property that exceed the length are truncated.
+    /// > `len(key + value) < u8::MAX`
+    ///
     /// `ip` can be one or more IP addresses, in a type that implements
     /// [`AsIpAddrs`] trait. It supports:
     ///
@@ -366,6 +369,19 @@ impl ServiceInfo {
             .cloned()
             .unwrap_or(ServiceStatus::Unknown)
     }
+
+    /// Consumes self and returns a resolved service, i.e. a lite version of `ServiceInfo`.
+    pub fn as_resolved_service(self) -> ResolvedService {
+        ResolvedService {
+            ty_domain: self.ty_domain,
+            sub_ty_domain: self.sub_domain,
+            fullname: self.fullname,
+            host: self.server,
+            port: self.port,
+            addresses: self.addresses,
+            txt_properties: self.txt_properties,
+        }
+    }
 }
 
 /// Removes potentially duplicated ".local." at the end of "hostname".
@@ -499,6 +515,26 @@ impl TxtProperties {
     pub fn get_property_val_str(&self, key: &str) -> Option<&str> {
         self.get(key).map(|x| x.val_str())
     }
+
+    /// Consumes properties and returns a hashmap, where the keys are the properties keys.
+    ///
+    /// If a property value is empty, return an empty string (because RFC 6763 allows empty values).
+    /// If a property value is non-empty but not valid UTF-8, skip the property and log a message.
+    pub fn into_property_map_str(self) -> HashMap<String, String> {
+        self.properties
+            .into_iter()
+            .filter_map(|property| {
+                let val_string = property.val.map_or(Some(String::new()), |val| {
+                    String::from_utf8(val)
+                        .map_err(|e| {
+                            debug!("Property value contains invalid UTF-8: {e}");
+                        })
+                        .ok()
+                })?;
+                Some((property.key, val_string))
+            })
+            .collect()
+    }
 }
 
 impl fmt::Display for TxtProperties {
@@ -681,6 +717,12 @@ where
     }
 }
 
+impl IntoTxtProperties for Vec<TxtProperty> {
+    fn into_txt_properties(self) -> TxtProperties {
+        TxtProperties { properties: self }
+    }
+}
+
 // Convert from properties key/value pairs to DNS TXT record content
 fn encode_txt<'a>(properties: impl Iterator<Item = &'a TxtProperty>) -> Vec<u8> {
     let mut bytes = Vec::new();
@@ -691,9 +733,16 @@ fn encode_txt<'a>(properties: impl Iterator<Item = &'a TxtProperty>) -> Vec<u8>
             s.extend(v);
         }
 
+        // Property that exceed the length are truncated
+        let sz: u8 = s.len().try_into().unwrap_or_else(|_| {
+            debug!("Property {} is too long, truncating to 255 bytes", prop.key);
+            s.resize(u8::MAX as usize, 0);
+            u8::MAX
+        });
+
         // TXT uses (Length,Value) format for each property,
         // i.e. the first byte is the length.
-        bytes.push(s.len().try_into().unwrap());
+        bytes.push(sz);
         bytes.extend(s);
     }
     if bytes.is_empty() {
@@ -757,7 +806,7 @@ fn decode_txt_unique(txt: &[u8]) -> Vec<TxtProperty> {
 }
 
 /// Returns true if `addr` is in the same network of `intf`.
-pub fn valid_ip_on_intf(addr: &IpAddr, intf: &Interface) -> bool {
+pub(crate) fn valid_ip_on_intf(addr: &IpAddr, intf: &Interface) -> bool {
     match (addr, &intf.addr) {
         (IpAddr::V4(addr), IfAddr::V4(intf)) => {
             let netmask = u32::from(intf.netmask);
@@ -1060,6 +1109,47 @@ pub(crate) fn split_sub_domain(domain: &str) -> (&str, Option<&str>) {
     }
 }
 
+/// Represents a resolved service as a plain data struct.
+/// This is from a client (i.e. querier) point of view.
+#[non_exhaustive]
+pub struct ResolvedService {
+    /// Service type and domain. For example, "_http._tcp.local."
+    pub ty_domain: String,
+
+    /// Optional service subtype and domain.
+    ///
+    /// See RFC6763 section 7.1 about "Subtypes":
+    /// <https://datatracker.ietf.org/doc/html/rfc6763#section-7.1>
+    /// For example, "_printer._sub._http._tcp.local."
+    pub sub_ty_domain: Option<String>,
+
+    /// Full name of the service. For example, "my-service._http._tcp.local."
+    pub fullname: String,
+
+    /// Host name of the service. For example, "my-server1.local."
+    pub host: String,
+
+    /// Port of the service. I.e. TCP or UDP port.
+    pub port: u16,
+
+    /// Addresses of the service. IPv4 or IPv6 addresses.
+    pub addresses: HashSet<IpAddr>,
+
+    /// Properties of the service, decoded from TXT record.
+    pub txt_properties: TxtProperties,
+}
+
+impl ResolvedService {
+    /// Returns true if the service data is valid, i.e. ready to be used.
+    pub fn is_valid(&self) -> bool {
+        let some_missing = self.ty_domain.is_empty()
+            || self.fullname.is_empty()
+            || self.host.is_empty()
+            || self.addresses.is_empty();
+        !some_missing
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::{
@@ -1127,6 +1217,21 @@ mod tests {
         assert_eq!(encoded.len(), "key6".len() + property_count);
         let decoded = decode_txt(&encoded);
         assert_eq!(properties, decoded);
+
+        // test very long property.
+        let properties = vec![TxtProperty::from(
+            String::from_utf8(vec![0x30; 1024]).unwrap().as_str(), // A long string of 0 char
+        )];
+        let property_count = properties.len();
+        let encoded = encode_txt(properties.iter());
+        assert_eq!(encoded.len(), 255 + property_count);
+        let decoded = decode_txt(&encoded);
+        assert_eq!(
+            vec![TxtProperty::from(
+                String::from_utf8(vec![0x30; 255]).unwrap().as_str()
+            )],
+            decoded
+        );
     }
 
     #[test]
