@@ -1,7 +1,7 @@
 use if_addrs::{IfAddr, Interface};
 use mdns_sd::{
     DaemonEvent, DaemonStatus, HostnameResolutionEvent, IfKind, IntoTxtProperties, ServiceDaemon,
-    ServiceEvent, ServiceInfo, UnregisterStatus,
+    ServiceEvent, ServiceInfo, TxtProperty, UnregisterStatus,
 };
 use std::collections::{HashMap, HashSet};
 use std::net::{IpAddr, Ipv4Addr, Ipv6Addr};
@@ -458,6 +458,29 @@ fn service_txt_properties_key_ascii() {
     assert!(my_service.is_ok());
 }
 
+#[test]
+fn test_txt_properties_into_hashmap_str() {
+    // Test valid UTF-8 properties
+    let properties = vec![("key1", "val1"), ("key2", "val2")].into_txt_properties();
+    let property_map = properties.into_property_map_str();
+    println!("property_map: {:?}", property_map);
+    assert_eq!(property_map.len(), 2);
+    assert_eq!(property_map.get("key1"), Some(&"val1".to_string()));
+    assert_eq!(property_map.get("key2"), Some(&"val2".to_string()));
+
+    // Test property with no value and property with invalid UTF-8
+    let invalid_vec: Vec<u8> = vec![200, 200]; // Invalid UTF-8 bytes
+    let prop1 = TxtProperty::from("key1");
+    let prop2 = TxtProperty::from(("key2", invalid_vec.as_slice()));
+    let properties = vec![prop1, prop2].into_txt_properties();
+    let property_map = properties.into_property_map_str();
+
+    // Property with no value should map to empty string
+    // Property with invalid UTF-8 should be skipped
+    assert_eq!(property_map.get("key1"), Some(&"".to_string()));
+    assert_eq!(property_map.len(), 1);
+}
+
 #[test]
 fn test_into_txt_properties() {
     // Verify (&str, String) tuple is supported.
@@ -475,6 +498,37 @@ fn test_into_txt_properties() {
     assert_eq!(txt_props.get_property_val_str("key2").unwrap(), "val2");
 }
 
+#[test]
+fn test_info_as_resolved_service() {
+    let sub_ty_domain = "_printer._sub._test._tcp.local.";
+    let service_info = ServiceInfo::new(
+        sub_ty_domain,
+        "my_instance",
+        "my_host.local.",
+        "192.168.0.1",
+        5200,
+        None,
+    )
+    .unwrap();
+    let resolved_service = service_info.as_resolved_service();
+    assert!(resolved_service.is_valid());
+    assert_eq!(resolved_service.sub_ty_domain.unwrap(), sub_ty_domain);
+    assert_eq!(resolved_service.ty_domain, "_test._tcp.local.");
+
+    let info_missing_addr = ServiceInfo::new(
+        "_test._tcp.local.",
+        "my_instance",
+        "my_host.local.",
+        "",
+        5200,
+        None,
+    )
+    .unwrap();
+    let invalid_service = info_missing_addr.as_resolved_service();
+    assert!(!invalid_service.is_valid());
+    assert!(invalid_service.sub_ty_domain.is_none());
+}
+
 /// Test enabling an interface using its name, for example "en0".
 /// Also tests an instance name with Upper Case.
 #[test]
@@ -626,6 +680,81 @@ fn service_with_ipv4_only() {
     d.shutdown().unwrap();
 }
 
+#[test]
+fn test_disable_interface_cache() {
+    // Create a server
+    let server = ServiceDaemon::new().expect("Failed to create the server");
+
+    // Register a service with one IPv4.
+    let ty_domain = "_disable-intf._tcp.local.";
+    let now = SystemTime::now()
+        .duration_since(SystemTime::UNIX_EPOCH)
+        .unwrap();
+    let instance_name = now.as_micros().to_string();
+    let service_ip_addr = my_ip_interfaces()
+        .iter()
+        .find(|iface| iface.ip().is_ipv4())
+        .map(|iface| iface.ip())
+        .unwrap();
+
+    let host_name = "disabled_intf_host.local.";
+    let port = 5201;
+    let my_service = ServiceInfo::new(
+        &ty_domain,
+        &instance_name,
+        host_name,
+        &service_ip_addr,
+        port,
+        None,
+    )
+    .expect("Invalid service info");
+    server
+        .register(my_service)
+        .expect("Failed to register our service");
+
+    // Create a client
+    let client = ServiceDaemon::new().expect("Failed to create the client");
+
+    // Give it some time to cache mDNS records.
+    sleep(Duration::from_secs(1));
+
+    // Disable the interface for the client.
+    client.disable_interface(service_ip_addr).unwrap();
+
+    // Browse for the service.
+    let handle = client.browse(&ty_domain).unwrap();
+    let timeout = Duration::from_secs(1);
+    let mut resolved = false;
+
+    // run till timeout and it should not resolve.
+    loop {
+        match handle.recv_timeout(timeout) {
+            Ok(event) => match event {
+                ServiceEvent::ServiceResolved(info) => {
+                    println!(
+                        "Resolved a service of {} addr(s): {:?}",
+                        &info.get_fullname(),
+                        info.get_addresses()
+                    );
+                    resolved = true;
+                    break;
+                }
+                _ => {}
+            },
+            Err(_) => {
+                break;
+            }
+        }
+    }
+
+    // We cannot resolve the service because the interface is disabled.
+    assert!(!resolved);
+
+    // Clean up.
+    server.shutdown().unwrap();
+    client.shutdown().unwrap();
+}
+
 #[test]
 fn service_with_invalid_addr_v4() {
     // Create a daemon
@@ -740,6 +869,76 @@ fn service_with_invalid_addr_v6() {
     assert!(!resolved);
 }
 
+#[test]
+fn service_with_loopback_addr() {
+    // Create a daemon
+    let d = ServiceDaemon::new().expect("Failed to create daemon");
+
+    d.enable_interface(IfKind::LoopbackV4)
+        .expect("Failed to enable loopback interface");
+
+    // Define a unique service type and instance name.
+    let ty_domain = "_test-loopback._tcp.local.";
+    let now = SystemTime::now()
+        .duration_since(SystemTime::UNIX_EPOCH)
+        .unwrap();
+    let instance_name = now.as_micros().to_string();
+
+    // Use a loopback address (127.0.0.1) for the service.
+    let loopback_ip: IpAddr = "127.0.0.1".parse().unwrap();
+    let host_name = "localhost.local.";
+    let port = 5201;
+    let my_service = ServiceInfo::new(
+        ty_domain,
+        &instance_name,
+        host_name,
+        loopback_ip,
+        port,
+        None,
+    )
+    .expect("valid service info");
+    d.register(my_service)
+        .expect("Failed to register our service");
+
+    // Browse for the service.
+    let browse_chan = d.browse(ty_domain).unwrap();
+    let timeout = Duration::from_secs(2);
+    let mut found_loopback = false;
+    loop {
+        match browse_chan.recv_timeout(timeout) {
+            Ok(event) => match event {
+                ServiceEvent::ServiceResolved(info) => {
+                    println!(
+                        "Resolved service {} with addresses: {:?}",
+                        info.get_fullname(),
+                        info.get_addresses()
+                    );
+                    // Check that at least one of the addresses is a loopback address.
+                    if info.get_addresses().iter().any(|ip| ip.is_loopback()) {
+                        found_loopback = true;
+                    }
+                    break;
+                }
+                e => {
+                    println!("Received event {:?}", e);
+                }
+            },
+            Err(e) => {
+                println!("browse error: {}", e);
+                break;
+            }
+        }
+    }
+
+    d.shutdown().unwrap();
+
+    // Assert that the resolved service includes a loopback address.
+    assert!(
+        found_loopback,
+        "The service should include a loopback address"
+    );
+}
+
 #[test]
 fn subtype() {
     // Create a daemon
@@ -1976,7 +2175,7 @@ fn test_multicast_loop_v6() {
     let server = ServiceDaemon::new().expect("failed to start server");
     server.set_multicast_loop_v6(false).unwrap();
 
-    // Get a single IPv4 address
+    // Get a single IPv6 address
     let ip_addr1 = my_ip_interfaces()
         .iter()
         .find(|iface| iface.ip().is_ipv6())
